package ui.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import classes.AbstractCpsObject;
import classes.CpsEdge;
import classes.CpsUpperNode;
import classes.Position;
import ui.model.Model;
import ui.view.UpperNodeCanvas;

public class NodeController {

	private Model model;
	private CanvasController cvs;
	private MultiPurposeController mpC;

	public NodeController(Model model, CanvasController cvs, MultiPurposeController mpC) {
		// TODO Auto-generated constructor stub
		this.model = model;
		this.cvs = cvs;
		this.mpC = mpC;
	}

	/**
	 * Add a CpsUpperNode into Canvas
	 * 
	 * @param nodeName
	 * @param upperNode
	 */
	public void doUpperNode(String nodeName, CpsUpperNode upperNode) {

		CpsUpperNode node = new CpsUpperNode(nodeName);
		node.setPosition(calculatePos());
		makeAdjacent(node, upperNode);
		makeNodeOfNodes(node, upperNode);
		backupOldEdges(node, upperNode);
		if (upperNode == null)
			cvs.addNewObject(node);
		else
			addObjectInUpperNode(node, upperNode);

	}

	/**
	 * Delete a CpsUpperNode from the Canvas
	 * 
	 * @param node
	 * @param upperNode
	 */
	public void undoUpperNode(CpsUpperNode node, CpsUpperNode upperNode) {
		restoreOldEdges(node, upperNode);
		unmakeNodesOfNodes(node, upperNode);
		unmakeAdjacent(node, upperNode);
		if (upperNode == null)
			cvs.deleteObjectOnCanvas(node);
		else
			deleteObjectInUpperNode(node, upperNode);
	}

	/**
	 * Put selected Nodes inside the Upper Node
	 * 
	 * @param node
	 * @param upperNode
	 */
	private void makeNodeOfNodes(CpsUpperNode node, CpsUpperNode upperNode) {

		ArrayList<AbstractCpsObject> nodes = node.getNodes();
		ArrayList<CpsEdge> toDelete = new ArrayList<>();

		// Put all selected Nodes into the Upper Node
		for (Iterator<AbstractCpsObject> it = model.getSelectedCpsObjects().iterator(); it.hasNext();) {

			// füge Neue Objecte in Neuen Node hinzu
			AbstractCpsObject obj = it.next();
			// POSITION MUSS NEU GEMACHT WERDEN FÜR OBJECTS IN NODE OF NODE
			// obj.setPosition((obj.getPosition().x + node.getPosition().x) / 2,
			// (obj.getPosition().y + node.getPosition().y) / 2);
			addObjectInUpperNode(obj, node);
		}

		// Füge die Edges die Alle Objekte vorher verbunden hat in neuen Node
		for (Iterator<CpsEdge> it = (upperNode == null ? model.getEdgesOnCanvas() : upperNode.getNodeEdges())
				.iterator(); it.hasNext();) {

			CpsEdge edge = it.next();

			// kopiere edge in UpperNode
			if (nodes.contains(edge.getA()) && nodes.contains(edge.getB())) {
				toDelete.add(edge);
				addEdge(edge, node);
			}

		}

		(upperNode == null ? model.getEdgesOnCanvas() : upperNode.getNodeEdges()).removeAll(toDelete);
		// Lösche dann die Edges aus der nächst höheren schicht
		for (AbstractCpsObject abs : model.getSelectedCpsObjects()) {
			if (upperNode == null)
				removeForNodeOfNode(abs, null);
			else
				removeForNodeOfNode(abs, upperNode);
		}

	}

	/**
	 * Transfer all relevant data from Node into the next higher layer of Node
	 * (upperNode)
	 * 
	 * @param node
	 * @param upperNode
	 */
	private void unmakeNodesOfNodes(CpsUpperNode node, CpsUpperNode upperNode) {
		// TODO Auto-generated method stub
		// add all nodes into upperNode
		(upperNode == null ? model.getObjectsOnCanvas() : upperNode.getNodes()).addAll(node.getNodes());
		// change the indices accordingly the higher layer
		mpC.adjustIdx(mpC.getHighestIdx((upperNode == null ? model.getCvsObjIdx() : upperNode.getNodesIdx())), node.getNodesIdx());
		// add all indices of nodes into upperNode
		(upperNode == null ? model.getCvsObjIdx() : upperNode.getNodesIdx()).putAll(node.getNodesIdx());
		// add all Edges of node into upperNode
		(upperNode == null ? model.getEdgesOnCanvas() : upperNode.getNodeEdges()).addAll(node.getNodeEdges());
	}

	/**
	 * Look for adjacent Nodes connected with the new CpsUpperNode and make
	 * Connections
	 * 
	 * @param node
	 * @param upperNode
	 */
	private void makeAdjacent(CpsUpperNode node, CpsUpperNode upperNode) {

		ArrayList<AbstractCpsObject> adj = new ArrayList<>();
		ArrayList<Float> maxCapacity = new ArrayList<>();

		ArrayList<CpsEdge> oldEdges = node.getOldEdges();
		ArrayList<CpsEdge> toDelete = new ArrayList<>();

		// für alle Edges auf dem Canvas
		for (Iterator<CpsEdge> it = (upperNode == null ? model.getEdgesOnCanvas() : upperNode.getNodeEdges())
				.iterator(); it.hasNext();) {

			CpsEdge edge = it.next();

			// wenn ausgewählte Objekte in edge und andere auserhalb
			if (model.getSelectedCpsObjects().contains(edge.getA())
					&& !model.getSelectedCpsObjects().contains(edge.getB())) {
				// und wenn in der adj noch nicht vorhanden und füg
				if (!adj.contains(edge.getB())) {
					adj.add(edge.getB());
					maxCapacity.add(edge.getCapacity());
				}
				// wenn vorhanden teste maxCapacity
				else if (maxCapacity.get(adj.indexOf(edge.getB())) < edge.getCapacity()) {
					maxCapacity.set(adj.indexOf(edge.getB()), edge.getCapacity());
				}
				// speichere alte edge
				oldEdges.add(edge);
				toDelete.add(edge);

			}

			// Analog
			else if (!model.getSelectedCpsObjects().contains(edge.getA())
					&& model.getSelectedCpsObjects().contains(edge.getB())) {
				if (!adj.contains(edge.getA())) {
					adj.add(edge.getA());
					maxCapacity.add(edge.getCapacity());
				} else if (maxCapacity.get(adj.indexOf(edge.getA())) < edge.getCapacity()) {
					maxCapacity.set(adj.indexOf(edge.getA()), edge.getCapacity());

				}
				oldEdges.add(edge);
				toDelete.add(edge);
			}
		}

		(upperNode == null ? model.getEdgesOnCanvas() : upperNode.getNodeEdges()).removeAll(toDelete);

		// für alle objekte in adjazenzliste mach
		for (AbstractCpsObject cps : adj) {
			CpsEdge newEdge = new CpsEdge(cps, node, maxCapacity.get(adj.indexOf(cps)));
			// !! Eventuell noch bei getConnections() hinzufügem
			// node.getConnections().add(newEdge);
			if (upperNode == null)
				cvs.addEdgeOnCanvas(newEdge);
			else
				upperNode.getNodeEdges().add(newEdge);

		}
	}

	/**
	 * Restore the Old Connections from the adjacent Object
	 * 
	 * @param node
	 * @param upperNode
	 */
	private void unmakeAdjacent(CpsUpperNode node, CpsUpperNode upperNode) {
		// TODO Auto-generated method stub
		ArrayList<CpsEdge> toDelete = new ArrayList<>();

		// für jede Edge aus upperNode die Node enthält tu sie in toDelete
		for (Iterator<CpsEdge> it = (upperNode == null ? model.getEdgesOnCanvas() : upperNode.getNodeEdges())
				.iterator(); it.hasNext();) {

			CpsEdge edge = it.next();
			if (edge.getA().equals(node) || edge.getB().equals(node))
				toDelete.add(edge);

		}
		// lösch alle Edges die in toDelete sind und wiederherstell alte Edges
		(upperNode == null ? model.getEdgesOnCanvas() : upperNode.getNodeEdges()).removeAll(toDelete);
		(upperNode == null ? model.getEdgesOnCanvas() : upperNode.getNodeEdges()).addAll(node.getOldEdges());
	}

	/**
	 * Backups Old Edges into next lower layer in Current UpperNode anymore
	 * 
	 * @param node
	 * @param upperNode
	 */
	private void backupOldEdges(CpsUpperNode node, CpsUpperNode upperNode) {

		ArrayList<CpsEdge> backup = new ArrayList<>();
		ArrayList<AbstractCpsObject> adj = new ArrayList<>();
		ArrayList<Float> maxCapacity = new ArrayList<>();

		// wenn upperNode in UpperNode erzeugt wird
		if (upperNode != null) {
			// für alle oldEdges die die zu gruppierenden nodes enthält werden
			// in die neuen upperNOde verschoben
			for (CpsEdge edge : upperNode.getOldEdges()) {
				if (!upperNode.getNodes().contains(edge.getA()) && !upperNode.getNodes().contains(edge.getB()))
					backup.add(edge);
			}
			// hier wird verschoben
			node.getOldEdges().addAll(backup);

			// neue adjazent muss hergestellt werden in den alten oldEdges
			for (CpsEdge edge : upperNode.getOldEdges()) {

				if (node.getNodes().contains(edge.getA()))
					if (!adj.contains(edge.getB())) {
						adj.add(edge.getB());
						maxCapacity.add(edge.getCapacity());
					} else if (maxCapacity.get(adj.indexOf(edge.getB())) < edge.getCapacity()) {
						maxCapacity.set(adj.indexOf(edge.getB()), edge.getCapacity());

					}
				if (node.getNodes().contains(edge.getB()))
					if (!adj.contains(edge.getA())) {
						adj.add(edge.getA());
						maxCapacity.add(edge.getCapacity());
					} else if (maxCapacity.get(adj.indexOf(edge.getA())) < edge.getCapacity()) {
						maxCapacity.set(adj.indexOf(edge.getA()), edge.getCapacity());

					}
			}
			// alle übertragenen Edges werden gelöscht
			upperNode.getOldEdges().removeAll(backup);
			// neue edges werden erzeugt
			for (AbstractCpsObject cps : adj)
				upperNode.getOldEdges().add(new CpsEdge(cps, node, maxCapacity.get(adj.indexOf(cps))));

		}
	}

	/**
	 * Restore Old Edges into next higher layer of UpperNode
	 * 
	 * @param node
	 * @param upperNode
	 */
	private void restoreOldEdges(CpsUpperNode node, CpsUpperNode upperNode) {

		ArrayList<CpsEdge> toDelete = new ArrayList<>();

		if (upperNode != null) {
			for (CpsEdge edge : node.getOldEdges()) {
				// wenn upperNode weder A oder B von von den OldEdges enthält
				if (!upperNode.getNodes().contains(edge.getA()) && !upperNode.getNodes().contains(edge.getB()))
					// Übertrage die OldEdges eine Schicht höher
					toDelete.add(edge);
			}
			// übertrage sie hier
			upperNode.getOldEdges().addAll(toDelete);
			// lösch sie aus den zu löschenden node, da sonst durch unmake
			// Adjacent probleme auftauchen
			node.getOldEdges().removeAll(toDelete);
			toDelete.clear();
			// lösche alle OldEdges aus der höheren schicht die zu den zu
			// löschenden Node vorher führten
			for (CpsEdge edge : upperNode.getOldEdges()) {
				if (edge.getA().equals(node) || edge.getB().equals(node))
					toDelete.add(edge);
			}
			upperNode.getOldEdges().removeAll(toDelete);
		}

	}

	/**
	 * Calculate new Position of the Upper Node
	 * 
	 * @return
	 */
	private Position calculatePos() {

		Position pos = new Position(0, 0);

		// sum(x0 .. xn) / numOfPos, y analog
		for (AbstractCpsObject abs : model.getSelectedCpsObjects()) {
			pos.x += abs.getPosition().x;
			pos.y += abs.getPosition().y;
		}
		pos.x /= model.getSelectedCpsObjects().size();
		pos.y /= model.getSelectedCpsObjects().size();

		return pos;
	}

	private void removeForNodeOfNode(AbstractCpsObject obj, CpsUpperNode upperNode) {

		mpC.decIdx(obj.getID(), (upperNode == null ? model.getCvsObjIdx() : upperNode.getNodesIdx()));
		(upperNode == null ? model.getCvsObjIdx() : upperNode.getNodesIdx()).remove(obj.getID());
		(upperNode == null ? model.getObjectsOnCanvas() : upperNode.getNodes()).remove(obj);
	}

	/**
	 * Add a Object into a CpsUpperNode
	 * 
	 * @param object
	 * @param upperNode
	 */
	public void addObjectInUpperNode(AbstractCpsObject object, CpsUpperNode upperNode) {
		object.setSav("UPPER" + upperNode.getID());
		upperNode.getNodesIdx().put(object.getID(), upperNode.getNodes().size());
		upperNode.getNodes().add(object);
	}

	/**
	 * Delete a AbstactCpsObject from CPSUpperNode
	 * 
	 * @param object
	 * @param upperNode
	 */
	public void deleteObjectInUpperNode(AbstractCpsObject object, CpsUpperNode upperNode) {
		CpsEdge e = null;
		for (AbstractCpsObject cps : upperNode.getNodes()) {
			for (CpsEdge p : cps.getConnections()) {
				if (p.getA() == object || p.getB() == object) {
					e = p;
				}
			}
			if (!model.getClipboradObjects().contains(cps) && !(cps instanceof CpsUpperNode)) {
				cps.getConnectedTo().remove(e);
			}
			upperNode.getNodeEdges().remove(e);
		}
		mpC.decIdx(object.getID(), upperNode.getNodesIdx());
		upperNode.getNodesIdx().remove(object.getID());
		upperNode.getNodes().remove(object);
	}

	/**
	 * Add Edge in UpperNode
	 * 
	 * @param edge
	 * @param upperNode
	 */
	public void addEdge(CpsEdge edge, CpsUpperNode upperNode) {
		upperNode.getNodeEdges().add(edge);
	}

	/**
	 * Delete Edge in UpperNode
	 * 
	 * @param edge
	 * @param upperNode
	 */
	public void deleteEdge(CpsEdge edge, CpsUpperNode upperNode) {
		edge.getA().getConnections().remove(edge);
		edge.getB().getConnections().remove(edge);
		upperNode.getNodeEdges().remove(edge);
		if (edge.getA() instanceof CpsUpperNode || edge.getB() instanceof CpsUpperNode)
			cvs.handleUpperNodes(edge);
	}

	/**
	 * Adds a Edge into Connections of AbstactCpsObjects
	 * 
	 * @param a
	 * @param b
	 */
	public void connectNodes(CpsEdge edge, CpsUpperNode upperNode) {
		// mache hier Connections in A und B rein
		// mache diese Edge auch in OldEdges rein des aktuellen CpsUpperNodes
		upperNode.getOldEdges().add(edge);
	}

	public void disconnectNodes(CpsEdge edge, CpsUpperNode upperNode) {
		edge.getA().getConnections().remove(edge);
		edge.getB().getConnections().remove(edge);
		if (upperNode != null)
			upperNode.getOldEdges().remove(edge);
	}

}
